/*************************************************************************
 inner2.c 

 Modified version of routines in inner.c. These routines attempt to be more
 generic so as to be reusable in any context. In particular their
 application is in the controls for the outer controls. Eventually we
 really need to ensure that we use the same inner control for both the
 mesh estimates and the mesh estimates for the outer control control.
 *************************************************************************/
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "path.h"
#include "option.h"
#include "mesh.h"
#include "blackScholes.h"
#include "control3.h"
#include "util.h"

/* weight macro now to replace teh weights array */
#define WEIGHT(time,prices_t,toPath) transProb(&(md->option.pData),prices_t,md->option.pData.price[time+1][toPath]) /md->distribution[(time)][(toPath)]


/*-----------------------------------------------------------------------
  The following set of routines are for the application of inner control 3.
  IControl1Correlation := computes the correlation b/w the two
*/
double IControl1Correlation(meshData *md, int time, double *prices,
       double *option_tp1){
  int max, toPath;
  double estimate,sumWeight,deflatedPrice;

  max = dWhichMax(md->option.pData.underliers, prices);

  estimate = 0;
  sumWeight = 0;
  for(toPath=0; toPath<md->meshPaths; toPath++){
    if(md->option.pData.price[time+1][toPath][max] >
md->option.strikePrice){
      deflatedPrice = exp(-md->option.riskFreeRate)*

(md->option.pData.price[time+1][toPath][max]-md->option.strikePrice);
    }else{
      deflatedPrice = 0;
    }
    estimate +=
WEIGHT(time,prices,toPath)*exp(-time*md->option.riskFreeRate)*
      deflatedPrice*option_tp1[toPath];
    sumWeight += WEIGHT(time,prices,toPath);
  }
  estimate /= sumWeight;
  return(estimate);
}


/*-----------------------------------------------------------------------
  The following set of routines are for the application of inner control 3.
  IControl3Correlation := computes the correlation b/w the two

  Control3 is the max of two from a given set of five underliers. Hence 
  we use an approach from Stulz to evalute this. The actual formula is from
  Kolb. See inner.c.
*/


double IControl3Correlation(meshData *md, int time, double *prices,
       double *option_tp1){
  int max, toPath;
  double estimate,deflatedPrice,weight,sumWeight;
  double e,p,r, maxPrice;

  int max1, max2, i;
  double maxval1, maxval2;

  /* At the current time find the max prices */

  maxval1 = 0.0;
  maxval2 = 0.0;
  max1 = dWhichMax(md->option.pData.underliers, prices);
  maxval1 = prices[max1];

  for(i=0; i<md->option.pData.underliers;i++){
    if(i != max1 && prices[i] > maxval2){
      max2 = i;
      maxval2 = prices[i];
    }
  }
  
  estimate = 0;
  sumWeight = 0;
  for(toPath=0; toPath<md->meshPaths; toPath++){
    if( md->option.pData.price[time+1][toPath][max1] >=
        md->option.pData.price[time+1][toPath][max2]){
      maxPrice = md->option.pData.price[time+1][toPath][max1];
    }else{
      maxPrice = md->option.pData.price[time+1][toPath][max2];
    }

    if(maxPrice > md->option.strikePrice){
      deflatedPrice = exp(-md->option.riskFreeRate)*
        (maxPrice - md->option.strikePrice);
    }else{
      deflatedPrice = 0;
    }
    estimate += WEIGHT(time,prices,toPath) * exp(-time*md->option.riskFreeRate)
      *deflatedPrice*option_tp1[toPath];
    sumWeight += WEIGHT(time,prices,toPath);
  }
  estimate /= sumWeight;
  return(estimate);
} 

double IMeshEstimate(meshData *md, int time, double *prices_t, 
        double *meshValue_tp1){
  int max, toPath;
  double estimate;
  double w,v;

  estimate = 0;
  for(toPath=0; toPath<md->meshPaths; toPath++){
    estimate +=  WEIGHT(time,prices_t,toPath)*meshValue_tp1[toPath];
  }
  return(estimate);
}

